@@ -17,17 +17,17 @@ Functions list
 +--------------------+---------------------------+------------------------------------------------------------+
 |                    | **Name**                  | **Description**                                            |
 +--------------------+---------------------------+------------------------------------------------------------+
-| Output Scope       | `set_scope`               | Create a new scope                                         |
+| Output Scope       | `put_scope`               | Create a new scope                                         |
 |                    +---------------------------+------------------------------------------------------------+
-|                    | `get_scope`               | Get the scope name in the runtime scope stack              |
+|                    | `use_scope`:sup:`†`       | Enter a scope                                              |
+|                    +---------------------------+------------------------------------------------------------+
+|                    | `get_scope`               | Get the current scope name in the runtime scope stack      |
 |                    +---------------------------+------------------------------------------------------------+
 |                    | `clear`                   | Clear the content of scope                                 |
 |                    +---------------------------+------------------------------------------------------------+
 |                    | `remove`                  | Remove the scope                                           |
 |                    +---------------------------+------------------------------------------------------------+
 |                    | `scroll_to`               | Scroll the page to the scope                               |
-|                    +---------------------------+------------------------------------------------------------+
-|                    | `use_scope`:sup:`†`       | Open or enter a scope                                      |
 +--------------------+---------------------------+------------------------------------------------------------+
 | Content Outputting | `put_text`                | Output plain text                                          |
 |                    +---------------------------+------------------------------------------------------------+
@@ -44,8 +44,6 @@ Functions list
 |                    +---------------------------+------------------------------------------------------------+
 |                    | `put_processbar`          | Output a process bar                                       |
 |                    +---------------------------+------------------------------------------------------------+
-|                    | `set_processbar`          | Set the progress of progress bar                           |
-|                    +---------------------------+------------------------------------------------------------+
 |                    | `put_loading`:sup:`†`     | Output loading prompt                                      |
 |                    +---------------------------+------------------------------------------------------------+
 |                    | `put_code`                | Output code block                                          |
@@ -85,8 +83,6 @@ Functions list
 |                    +---------------------------+------------------------------------------------------------+
 |                    | `style`:sup:`*`           | Customize the css style of output content                  |
 +--------------------+---------------------------+------------------------------------------------------------+
-| Placeholder        | `output`:sup:`*`          | Placeholder of output                                      |
-+--------------------+---------------------------+------------------------------------------------------------+
 
 Output Scope
 --------------
@@ -95,12 +91,12 @@ Output Scope
 
    * :ref:`Use Guide: Output Scope <output_scope>`
 
-.. autofunction:: set_scope
+.. autofunction:: put_scope
+.. autofunction:: use_scope
 .. autofunction:: get_scope
 .. autofunction:: clear
 .. autofunction:: remove
 .. autofunction:: scroll_to
-.. autofunction:: use_scope
 
 Content Outputting
 -----------------------
@@ -169,7 +165,7 @@ index equal ``position``:
     Output Messages.
 
     :param contents: Message contents.
-       The item is ``put_xxx()`` call, and any other type will be coverted to ``put_text(content)``.
+       The item is ``put_xxx()`` call, and any other type will be converted to ``put_text(content)``.
     :param bool closable: Whether to show a dismiss button on the right of the message.
     :param int scope, position: Those arguments have the same meaning as for `put_text()`
 
@@ -207,11 +203,9 @@ Layout and Style
 .. autofunction:: put_grid
 .. autofunction:: style
 
-Placeholder
---------------
-.. autofunction:: output
 
 """
+import copy
 import html
 import io
 import logging
@@ -232,7 +226,7 @@ except ImportError:
 
 logger = logging.getLogger(__name__)
 
-__all__ = ['Position', 'remove', 'scroll_to', 'put_tabs',
+__all__ = ['Position', 'remove', 'scroll_to', 'put_tabs', 'put_scope',
            'put_text', 'put_html', 'put_code', 'put_markdown', 'use_scope', 'set_scope', 'clear', 'remove',
            'put_table', 'put_buttons', 'put_image', 'put_file', 'PopupSize', 'popup', 'put_button',
            'close_popup', 'put_widget', 'put_collapse', 'put_link', 'put_scrollable', 'style', 'put_column',
@@ -407,7 +401,7 @@ def put_info(*contents, closable=False, scope=None, position=OutputPosition.BOTT
     """Output information message.
 
     :param contents: Message contents.
-       The item is ``put_xxx()`` call, and any other type will be coverted to ``put_text(content)``.
+       The item is ``put_xxx()`` call, and any other type will be converted to ``put_text(content)``.
     :param bool closable: Whether to show a dismiss button on the right of the message.
     :param int scope, position: Those arguments have the same meaning as for `put_text()`
 
@@ -483,14 +477,41 @@ def put_code(content, language='', rows=None, scope=None, position=OutputPositio
     return out
 
 
-def put_markdown(mdcontent, strip_indent=0, lstrip=False, options=None, sanitize=True,
-                 scope=None, position=OutputPosition.BOTTOM) -> Output:
+def _left_strip_multiple_line_string_literal(s):
+    """Remove the indent for code format in string literal
+
+    * The first line may have no leading whitespace
+    * There may be empty line in s (since PyCharm will remove the line trailing whitespace)
+    """
+    lines = s.splitlines()
+    if len(lines) < 2:
+        return s
+
+    line = ''
+    for line in lines[1:]:
+        if line:
+            break
+
+    strip_cnt = 1
+    while line[:strip_cnt] in (' ' * strip_cnt, '\t' * strip_cnt):
+        strip_cnt += 1
+
+    for line in lines[1:]:
+        while line.strip() and line[:strip_cnt] not in (' ' * strip_cnt, '\t' * strip_cnt):
+            strip_cnt -= 1
+
+    lines_ = [i[strip_cnt:] for i in lines[1:]]
+    return '\n'.join(lines[:1] + lines_)
+
+
+def put_markdown(mdcontent, lstrip=True, options=None, sanitize=True,
+                 scope=None, position=OutputPosition.BOTTOM, **kwargs) -> Output:
     """
     Output Markdown
 
     :param str mdcontent: Markdown string
-    :param int strip_indent: For each line of ``mdcontent``, if the first ``strip_indent`` characters are spaces, remove them
-    :param bool lstrip: Whether to remove the whitespace at the beginning of each line of ``mdcontent``
+    :param bool lstrip: Whether to remove the leading whitespace in each line of ``mdcontent``.
+        The number of the whitespace to remove will be decided cleverly.
     :param dict options: Configuration when parsing Markdown.
        PyWebIO uses `marked <https://marked.js.org/>`_ library to parse Markdown,
        the parse options see: https://marked.js.org/using_advanced#options (Only supports members of string and boolean type)
@@ -498,36 +519,26 @@ def put_markdown(mdcontent, strip_indent=0, lstrip=False, options=None, sanitize
     :param int scope, position: Those arguments have the same meaning as for `put_text()`
 
     When using Python triple quotes syntax to output multi-line Markdown in a function,
-    if you indent the Markdown text, you can use ``strip_indent`` or ``lstrip`` to prevent Markdown from parsing errors
-    (But do not use ``strip_indent`` and ``lstrip`` at the same time)::
+    you can indent the Markdown text to keep a good code format.
+    PyWebIO will cleverly remove the indent for you when show the Markdown::
 
-        # It is ugly without strip_indent or lstrip
-        def hello():
-            put_markdown(r\""" # H1
-        This is content.
-        \""")
-
-        # Using lstrip to get beautiful indent
+        # good code format
         def hello():
             put_markdown(r\""" # H1
             This is content.
-            \""", lstrip=True)
+            \""")
 
-        # Using strip_indent to get beautiful indent
-        def hello():
-            put_markdown(r\""" # H1
-            This is content.
-            \""", strip_indent=4)
+    .. versionchanged:: 1.5
+       Enable `lstrip` by default.
+       Deprecate `strip_indent`.
     """
-    if strip_indent:
-        lines = (
-            i[strip_indent:] if (i[:strip_indent] == ' ' * strip_indent) else i
-            for i in mdcontent.splitlines()
-        )
-        mdcontent = '\n'.join(lines)
+    if 'strip_indent' in kwargs:
+        import warnings
+        # use stacklevel=2 to make the warning refer to put_markdown() call
+        warnings.warn("`strip_indent` parameter is deprecated in `put_markdown()`", DeprecationWarning, stacklevel=2)
+
     if lstrip:
-        lines = (i.lstrip() for i in mdcontent.splitlines())
-        mdcontent = '\n'.join(lines)
+        mdcontent = _left_strip_multiple_line_string_literal(mdcontent)
 
     spec = _get_output_spec('markdown', content=mdcontent, options=options, sanitize=sanitize,
                             scope=scope, position=position)
@@ -664,17 +675,19 @@ def put_table(tdata, header=None, scope=None, position=OutputPosition.BOTTOM) ->
 
 def _format_button(buttons):
     """
-    Format `buttons` parameter in `put_buttons()`
+    Format `buttons` parameter in `put_buttons()`, replace its value with its idx
     :param buttons:
         {label:, value:, }
         (label, value, )
         single value, label=value
 
-    :return: [{value:, label:, }, ...]
+    :return: [{value:, label:, }, ...], values
     """
 
     btns = []
-    for btn in buttons:
+    values = []
+    for idx, btn in enumerate(buttons):
+        btn = copy.deepcopy(btn)
         if isinstance(btn, Mapping):
             assert 'value' in btn and 'label' in btn, 'actions item must have value and label key'
         elif isinstance(btn, (list, tuple)):
@@ -682,8 +695,10 @@ def _format_button(buttons):
             btn = dict(zip(('label', 'value'), btn))
         else:
             btn = dict(value=btn, label=btn)
+        values.append(btn['value'])
+        btn['value'] = idx
         btns.append(btn)
-    return btns
+    return btns, values
 
 
 def put_buttons(buttons, onclick, small=None, link_style=False, outline=False, group=False, scope=None,
@@ -693,11 +708,19 @@ def put_buttons(buttons, onclick, small=None, link_style=False, outline=False, g
 
     :param list buttons: Button list. The available formats of list items are:
 
-        * dict: ``{label:(str)button label, value:(str)button value, color:(str, optional)button color}``
+        * dict::
+
+            {
+                "label":(str)button label,
+                "value":(str)button value,
+                "color":(str, optional)button color,
+                "disabled":(bool, optional) whether the button is disabled
+            }
+
         * tuple or list: ``(label, value)``
         * single value: label and value of option use the same value
 
-        The ``value`` of button can be any JSON serializable object.
+        The ``value`` of button can be any type.
         The ``color`` of button can be one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`.
 
         Example:
@@ -763,18 +786,23 @@ def put_buttons(buttons, onclick, small=None, link_style=False, outline=False, g
             put_text("You click delete button")
 
         put_buttons(['edit', 'delete'], onclick=[edit, delete])
+
+    .. versionchanged:: 1.5
+       Add ``disabled`` button support.
+       The ``value`` of button can be any object.
     """
-    btns = _format_button(buttons)
+    btns, values = _format_button(buttons)
 
     if isinstance(onclick, Sequence):
         assert len(btns) == len(onclick), "`onclick` and `buttons` must be same length."
-        onclick = {btn['value']: callback for btn, callback in zip(btns, onclick)}
 
-    def click_callback(btn_val):
-        if isinstance(onclick, dict):
-            return onclick[btn_val]()
+    def click_callback(btn_idx):
+        if isinstance(onclick, Sequence):
+            return onclick[btn_idx]()
         else:
-            return onclick(btn_val)
+            btn_val = values[btn_idx]
+            if not btns[btn_idx].get('disabled'):
+                return onclick(btn_val)
 
     callback_id = output_register_callback(click_callback, **callback_options)
     spec = _get_output_spec('buttons', callback_id=callback_id, buttons=btns, small=small,
@@ -783,7 +811,7 @@ def put_buttons(buttons, onclick, small=None, link_style=False, outline=False, g
     return Output(spec)
 
 
-def put_button(label, onclick, color=None, small=None, link_style=False, outline=False, scope=None,
+def put_button(label, onclick, color=None, small=None, link_style=False, outline=False, disabled=False, scope=None,
                position=OutputPosition.BOTTOM) -> Output:
     """Output a single button and bind click event to it.
 
@@ -791,6 +819,7 @@ def put_button(label, onclick, color=None, small=None, link_style=False, outline
     :param callable onclick: Callback which will be called when button is clicked.
     :param str color: The color of the button,
         can be one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`.
+    :param bool disabled: Whether the button is disabled
     :param - small, link_style, outline, scope, position:  Those arguments have the same meaning as for `put_buttons()`
 
     Example:
@@ -802,9 +831,13 @@ def put_button(label, onclick, color=None, small=None, link_style=False, outline
         put_button("click me", onclick=lambda: toast("Clicked"), color='success', outline=True)
 
     .. versionadded:: 1.4
+
+    .. versionchanged:: 1.5
+       add ``disabled`` parameter
     """
-    return put_buttons([{'label': label, 'value': '', 'color': color or 'primary'}], onclick=[onclick],
-                       small=small, link_style=link_style, outline=outline, scope=scope, position=position)
+    return put_buttons([{'label': label, 'value': '', 'color': color or 'primary', 'disabled': disabled}],
+                       onclick=[onclick], small=small, link_style=link_style, outline=outline, scope=scope,
+                       position=position)
 
 
 def put_image(src, format=None, title='', width=None, height=None,
@@ -859,7 +892,7 @@ def put_file(name, content, label=None, scope=None, position=OutputPosition.BOTT
 
     To show a link with the file name on the browser. When click the link, the browser automatically downloads the file.
 
-    :param str name: File name when downloading
+    :param str name: File name downloaded as
     :param content: File content. It is a bytes-like object
     :param str label: The label of the download link, which is the same as the file name by default.
     :param int scope, position: Those arguments have the same meaning as for `put_text()`
@@ -870,7 +903,9 @@ def put_file(name, content, label=None, scope=None, position=OutputPosition.BOTT
         :name: put_file
         :summary: `put_file()` usage
 
-        put_file('hello-world.txt', b'hello world!', 'download me')
+        content = open('./some-file', 'rb').read()  # ..doc-only
+        content = open('README.md', 'rb').read()    # ..demo-only
+        put_file('hello-world.txt', content, 'download me')
     """
     if label is None:
         label = name
@@ -923,6 +958,8 @@ def put_processbar(name, init=0, label=None, auto_close=False, scope=None,
         for i in range(1, 11):
             set_processbar('bar', i / 10)
             time.sleep(0.1)
+
+    .. seealso:: use `set_processbar()` to set the progress of progress bar
     """
     processbar_id = 'webio-processbar-%s' % name
     percentage = init * 100
@@ -1061,7 +1098,7 @@ def put_collapse(title, content=[], open=False, scope=None, position=OutputPosit
 
 
 @safely_destruct_output_when_exp('content')
-def put_scrollable(content=[], height=400, keep_bottom=False, horizon_scroll=False, border=True,
+def put_scrollable(content=[], height=400, keep_bottom=False, border=True,
                    scope=None, position=OutputPosition.BOTTOM, **kwargs) -> Output:
     """Output a fixed height content area. scroll bar is displayed when the content exceeds the limit
 
@@ -1070,7 +1107,8 @@ def put_scrollable(content=[], height=400, keep_bottom=False, horizon_scroll=Fal
     :param int/tuple height: The height of the area (in pixels).
        ``height`` parameter also accepts ``(min_height, max_height)`` to indicate the range of height, for example,
        ``(100, 200)`` means that the area has a minimum height of 100 pixels and a maximum of 200 pixels.
-    :param bool horizon_scroll: Whether to use the horizontal scroll bar
+       Set ``None`` if you don't want to limit the height
+    :param bool keep_bottom: Whether to keep the content area scrolled to the bottom when updated.
     :param bool border: Whether to show border
     :param int scope, position: Those arguments have the same meaning as for `put_text()`
 
@@ -1082,16 +1120,19 @@ def put_scrollable(content=[], height=400, keep_bottom=False, horizon_scroll=Fal
 
         import time
 
-        o = output("You can click the area to prevent auto scroll.")
-        put_scrollable(o, height=200, keep_bottom=True)
+        put_scrollable(put_scope('scrollable'), height=200, keep_bottom=True)
+        put_text("You can click the area to prevent auto scroll.", scope='scrollable')
 
         while 1:
-            o.append(time.time())
+            put_text(time.time(), scope='scrollable')
             time.sleep(0.5)
 
     .. versionchanged:: 1.1
        add ``height`` parameter，remove ``max_height`` parameter；
        add ``keep_bottom`` parameter
+
+    .. versionchanged:: 1.5
+       remove ``horizon_scroll`` parameter
     """
     if not isinstance(content, (list, tuple, OutputList)):
         content = [content]
@@ -1100,8 +1141,9 @@ def put_scrollable(content=[], height=400, keep_bottom=False, horizon_scroll=Fal
 
     if 'max_height' in kwargs:
         import warnings
+        # use stacklevel=2 to make the warning refer to the put_scrollable() call
         warnings.warn("`max_height` parameter is deprecated in `put_scrollable()`, use `height` instead.",
-                      DeprecationWarning, stacklevel=3)
+                      DeprecationWarning, stacklevel=2)
         height = kwargs['max_height']  # Backward compatible
 
     try:
@@ -1109,41 +1151,9 @@ def put_scrollable(content=[], height=400, keep_bottom=False, horizon_scroll=Fal
     except Exception:
         min_height, max_height = height, height
 
-    dom_id = 'pywebio-%s' % random_str(10)
-
-    tpl = """<div id="{{dom_id}}" {{#keep_bottom}}tabindex="0"{{/keep_bottom}}
-        style="min-height: {{min_height}}px; max-height: {{max_height}}px;
-            overflow-y: scroll;
-            {{#horizon_scroll}}overflow-x: scroll;{{/horizon_scroll}}
-            {{#border}} 
-            border: 1px solid rgba(0,0,0,.125);
-            box-shadow: inset 0 0 2px 0 rgba(0,0,0,.1); 
-            {{/border}}
-            padding: 10px;
-            margin-bottom: 10px;">
-
-        {{#contents}}
-            {{& pywebio_output_parse}}
-        {{/contents}}
-    </div>"""
-    if keep_bottom:
-        tpl += """
-        <script>
-            (function(){
-                let div = document.getElementById(%r), stop=false;
-                $(div).on('focusin', function(e){ stop=true }).on('focusout', function(e){ stop=false });;
-                new MutationObserver(function (mutations, observe) {
-                    if(!stop) $(div).stop().animate({ scrollTop: $(div).prop("scrollHeight")}, 200);
-                }).observe(div, { childList: true, subtree:true });
-            })();
-        </script>
-        """ % dom_id
-
-    return put_widget(template=tpl,
-                      data=dict(dom_id=dom_id, contents=content, min_height=min_height,
-                                max_height=max_height, keep_bottom=keep_bottom,
-                                horizon_scroll=horizon_scroll, border=border),
-                      scope=scope, position=position).enable_context_manager()
+    spec = _get_output_spec('scrollable', contents=content, min_height=min_height, max_height=max_height,
+                            keep_bottom=keep_bottom, border=border, scope=scope, position=position)
+    return Output(spec).enable_context_manager(container_selector='> div')
 
 
 @safely_destruct_output_when_exp('tabs')
@@ -1346,7 +1356,8 @@ def put_grid(content, cell_width='auto', cell_height='auto', cell_widths=None, c
         for y in range(len(content[x])):
             cell = content[x][y]
             if isinstance(cell, span_):
-                for i in range(cell.row): lens[x + i] += cell.col
+                for i in range(cell.row):
+                    lens[x + i] += cell.col
 
                 css = 'grid-row-start: span {row}; grid-column-start: span {col};'.format(row=cell.row, col=cell.col)
                 elem = put_html('<div></div>') if cell.content is None else cell.content
@@ -1388,15 +1399,36 @@ def put_grid(content, cell_width='auto', cell_height='auto', cell_widths=None, c
     return put_widget(template=tpl, data=dict(contents=content), scope=scope, position=position)
 
 
+@safely_destruct_output_when_exp('content')
+def put_scope(name, content=[], scope=None, position=OutputPosition.BOTTOM) -> Output:
+    """Output a scope
+
+    :param str name:
+    :param list/put_xxx() content: The initial content of the scope, can be ``put_xxx()`` or a list of it.
+    :param int scope, position: Those arguments have the same meaning as for `put_text()`
+    """
+    if not isinstance(content, list):
+        content = [content]
+
+    assert is_html_safe_value(name), "Scope name only allow letter/digit/'_'/'-' char."
+    dom_id = scope2dom(name, no_css_selector=True)
+
+    spec = _get_output_spec('scope', dom_id=dom_id, contents=content, scope=scope, position=position)
+    return Output(spec)
+
+
 @safely_destruct_output_when_exp('contents')
 def output(*contents):
     """Placeholder of output
 
+    .. deprecated:: 1.5
+        See :ref:`User Guide <put_scope>` for new way to set css style for output.
+
      ``output()`` can be passed in anywhere that ``put_xxx()`` can passed in. A handler it returned by ``output()``,
      and after being output, the content can also be modified by the handler (See code example below).
 
     :param contents: The initial contents to be output.
-       The item is ``put_xxx()`` call, and any other type will be coverted to ``put_text(content)``.
+       The item is ``put_xxx()`` call, and any other type will be converted to ``put_text(content)``.
     :return: An OutputHandler instance, the methods of the instance are as follows:
 
     * ``reset(*contents)`` : Reset original contents to ``contents``
@@ -1429,6 +1461,11 @@ def output(*contents):
 
     """
 
+    import warnings
+    # use stacklevel=2 to make the warning refer to the output() call
+    warnings.warn("`pywebio.output.output()` is deprecated since v1.5 and will remove in the future version, "
+                  "use `pywebio.output.put_scope()` instead", DeprecationWarning, stacklevel=2)
+
     class OutputHandler(Output):
         """
         与 `Output` 的不同在于， 不会在销毁时(__del__)自动输出
@@ -1685,17 +1722,16 @@ def toast(content, duration=2, position='center', color='info', onclick=None):
 clear_scope = clear
 
 
-def use_scope(name=None, clear=False, create_scope=True, **scope_params):
-    """Open or enter a scope. Can be used as context manager and decorator.
+def use_scope(name=None, clear=False, **kwargs):
+    """use_scope(name=None, clear=False)
+
+    Open or enter a scope. Can be used as context manager and decorator.
 
     See :ref:`User manual - use_scope() <use_scope>`
 
     :param str name: Scope name. If it is None, a globally unique scope name is generated.
         (When used as context manager, the context manager will return the scope name)
     :param bool clear: Whether to clear the contents of the scope before entering the scope.
-    :param bool create_scope: Whether to create scope when scope does not exist.
-    :param scope_params: Extra parameters passed to `set_scope()` when need to create scope.
-        Only available when ``create_scope=True``.
 
     :Usage:
 
@@ -1709,6 +1745,13 @@ def use_scope(name=None, clear=False, create_scope=True, **scope_params):
             put_xxx()
 
     """
+    # For backward compatible
+    #     :param bool create_scope: Whether to create scope when scope does not exist.
+    #     :param scope_params: Extra parameters passed to `set_scope()` when need to create scope.
+    #         Only available when ``create_scope=True``.
+    create_scope = kwargs.pop('create_scope', True)
+    scope_params = kwargs
+
     if name is None:
         name = random_str(10)
     else:
@@ -1716,10 +1759,8 @@ def use_scope(name=None, clear=False, create_scope=True, **scope_params):
 
     def before_enter():
         if create_scope:
-            set_scope(name, **scope_params)
-
-        if clear:
-            clear_scope(name)
+            if_exist = 'clear' if clear else None
+            set_scope(name, if_exist=if_exist, **scope_params)
 
     return use_scope_(name=name, before_enter=before_enter)
 
