@@ -8,12 +8,13 @@ from tornado import template
 from tornado.web import HTTPError, Finish
 from tornado.web import StaticFileHandler
 
-from . import utils
+from . import page
 from .httpbased import HttpHandler
 from .tornado import webio_handler, set_ioloop
 from .tornado_http import TornadoHttpContext
-from .utils import cdn_validation, make_applications
-from ..session import register_session_implement, CoroutineBasedSession, ThreadBasedSession
+from .utils import cdn_validation, print_listen_address
+from .page import make_applications
+from ..session import register_session_implement, CoroutineBasedSession, ThreadBasedSession, Session
 from ..utils import get_free_port, STATIC_PATH, parse_file_size
 
 
@@ -209,7 +210,7 @@ def _path_deploy(base, port=0, host='', static_dir=None, cdn=True, max_payload_s
         handlers.append((r"/_pywebio_static/(.*)", StaticFileHandler, {"path": STATIC_PATH}))
     handlers.append((r"/.*", RequestHandler))
 
-    print('Listen on %s:%s' % (host or '0.0.0.0', port))
+    print_listen_address(host, port)
 
     set_ioloop(tornado.ioloop.IOLoop.current())  # to enable bokeh app
     app = tornado.web.Application(handlers=handlers, **tornado_app_settings)
@@ -245,8 +246,8 @@ def path_deploy(base, port=0, host='',
 
     The rest arguments of ``path_deploy()`` have the same meaning as for :func:`pywebio.platform.tornado.start_server`
     """
-
-    utils.MAX_PAYLOAD_SIZE = max_payload_size = parse_file_size(max_payload_size)
+    debug = Session.debug = os.environ.get('PYWEBIO_DEBUG', debug)
+    page.MAX_PAYLOAD_SIZE = max_payload_size = parse_file_size(max_payload_size)
     # Since some cloud server may close idle connections (such as heroku),
     # use `websocket_ping_interval` to  keep the connection alive
     tornado_app_settings.setdefault('websocket_ping_interval', 30)
@@ -301,7 +302,9 @@ def path_deploy_http(base, port=0, host='',
 
     The rest arguments of ``path_deploy_http()`` have the same meaning as for :func:`pywebio.platform.tornado_http.start_server`
     """
-    utils.MAX_PAYLOAD_SIZE = max_payload_size = parse_file_size(max_payload_size)
+    debug = Session.debug = os.environ.get('PYWEBIO_DEBUG', debug)
+
+    page.MAX_PAYLOAD_SIZE = max_payload_size = parse_file_size(max_payload_size)
 
     gen = _path_deploy(base, port=port, host=host,
                        static_dir=static_dir,
